@@ -27,35 +27,35 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <stdlib.h>
 
 #include "common/intops.h"
 
 #include "src/cdef.h"
+#include "src/tables.h"
 
 static inline int constrain(const int diff, const int threshold,
-                            const int damping)
+                            const int shift)
 {
-    if (!threshold) return 0;
-    const int shift = imax(0, damping - ulog2(threshold));
-    return apply_sign(imin(abs(diff), imax(0, threshold - (abs(diff) >> shift))),
-                      diff);
+    const int adiff = abs(diff);
+    return apply_sign(imin(adiff, imax(0, threshold - (adiff >> shift))), diff);
 }
 
-static inline void fill(uint16_t *tmp, const ptrdiff_t stride,
+static inline void fill(int16_t *tmp, const ptrdiff_t stride,
                         const int w, const int h)
 {
+    /* Use a value that's a large positive number when interpreted as unsigned,
+     * and a large negative number when interpreted as signed. */
     for (int y = 0; y < h; y++) {
         for (int x = 0; x < w; x++)
-            tmp[x] = INT16_MAX;
+            tmp[x] = INT16_MIN;
         tmp += stride;
     }
 }
 
-static void padding(uint16_t *tmp, const ptrdiff_t tmp_stride,
+static void padding(int16_t *tmp, const ptrdiff_t tmp_stride,
                     const pixel *src, const ptrdiff_t src_stride,
-                    const pixel (*left)[2], pixel *const top[2],
+                    const pixel (*left)[2], const pixel *top,
                     const int w, const int h,
                     const enum CdefEdgeFlags edges)
 {
@@ -78,9 +78,11 @@ static void padding(uint16_t *tmp, const ptrdiff_t tmp_stride,
         x_end -= 2;
     }
 
-    for (int y = y_start; y < 0; y++)
+    for (int y = y_start; y < 0; y++) {
         for (int x = x_start; x < x_end; x++)
-            tmp[x + y * tmp_stride] = top[y & 1][x];
+            tmp[x + y * tmp_stride] = top[x];
+        top += PXSTRIDE(src_stride);
+    }
     for (int y = 0; y < h; y++)
         for (int x = x_start; x < 0; x++)
             tmp[x + y * tmp_stride] = left[y][2 + x];
@@ -94,75 +96,113 @@ static void padding(uint16_t *tmp, const ptrdiff_t tmp_stride,
 
 static NOINLINE void
 cdef_filter_block_c(pixel *dst, const ptrdiff_t dst_stride,
-                    const pixel (*left)[2], /*const*/ pixel *const top[2],
-                    const int w, const int h, const int pri_strength,
-                    const int sec_strength, const int dir,
-                    const int damping, const enum CdefEdgeFlags edges
-                    HIGHBD_DECL_SUFFIX)
+                    const pixel (*left)[2], const pixel *const top,
+                    const int pri_strength, const int sec_strength,
+                    const int dir, const int damping, const int w, int h,
+                    const enum CdefEdgeFlags edges HIGHBD_DECL_SUFFIX)
 {
-    static const int8_t cdef_directions[8 /* dir */][2 /* pass */] = {
-        { -1 * 12 + 1, -2 * 12 + 2 },
-        {  0 * 12 + 1, -1 * 12 + 2 },
-        {  0 * 12 + 1,  0 * 12 + 2 },
-        {  0 * 12 + 1,  1 * 12 + 2 },
-        {  1 * 12 + 1,  2 * 12 + 2 },
-        {  1 * 12 + 0,  2 * 12 + 1 },
-        {  1 * 12 + 0,  2 * 12 + 0 },
-        {  1 * 12 + 0,  2 * 12 - 1 }
-    };
     const ptrdiff_t tmp_stride = 12;
     assert((w == 4 || w == 8) && (h == 4 || h == 8));
-    uint16_t tmp_buf[144];  // 12*12 is the maximum value of tmp_stride * (h + 4)
-    uint16_t *tmp = tmp_buf + 2 * tmp_stride + 2;
-    const int bitdepth_min_8 = bitdepth_from_max(bitdepth_max) - 8;
-    const int pri_tap = 4 - ((pri_strength >> bitdepth_min_8) & 1);
+    int16_t tmp_buf[144]; // 12*12 is the maximum value of tmp_stride * (h + 4)
+    int16_t *tmp = tmp_buf + 2 * tmp_stride + 2;
 
     padding(tmp, tmp_stride, dst, dst_stride, left, top, w, h, edges);
 
-    // run actual filter
-    for (int y = 0; y < h; y++) {
-        for (int x = 0; x < w; x++) {
-            int sum = 0;
-            const int px = dst[x];
-            int max = px, min = px;
-            int pri_tap_k = pri_tap;
-            for (int k = 0; k < 2; k++) {
-                const int off1 = cdef_directions[dir][k];
-                const int p0 = tmp[x + off1];
-                const int p1 = tmp[x - off1];
-                sum += pri_tap_k * constrain(p0 - px, pri_strength, damping);
-                sum += pri_tap_k * constrain(p1 - px, pri_strength, damping);
-                // if pri_tap_k == 4 then it becomes 2 else it remains 3
-                pri_tap_k -= (pri_tap_k << 1) - 6;
-                if (p0 != INT16_MAX) max = imax(p0, max);
-                if (p1 != INT16_MAX) max = imax(p1, max);
-                min = imin(p0, min);
-                min = imin(p1, min);
-                const int off2 = cdef_directions[(dir + 2) & 7][k];
-                const int s0 = tmp[x + off2];
-                const int s1 = tmp[x - off2];
-                const int off3 = cdef_directions[(dir + 6) & 7][k];
-                const int s2 = tmp[x + off3];
-                const int s3 = tmp[x - off3];
-                if (s0 != INT16_MAX) max = imax(s0, max);
-                if (s1 != INT16_MAX) max = imax(s1, max);
-                if (s2 != INT16_MAX) max = imax(s2, max);
-                if (s3 != INT16_MAX) max = imax(s3, max);
-                min = imin(s0, min);
-                min = imin(s1, min);
-                min = imin(s2, min);
-                min = imin(s3, min);
-                // sec_tap starts at 2 and becomes 1
-                const int sec_tap = 2 - k;
-                sum += sec_tap * constrain(s0 - px, sec_strength, damping);
-                sum += sec_tap * constrain(s1 - px, sec_strength, damping);
-                sum += sec_tap * constrain(s2 - px, sec_strength, damping);
-                sum += sec_tap * constrain(s3 - px, sec_strength, damping);
-            }
-            dst[x] = iclip(px + ((8 + sum - (sum < 0)) >> 4), min, max);
+    if (pri_strength) {
+        const int bitdepth_min_8 = bitdepth_from_max(bitdepth_max) - 8;
+        const int pri_tap = 4 - ((pri_strength >> bitdepth_min_8) & 1);
+        const int pri_shift = imax(0, damping - ulog2(pri_strength));
+        if (sec_strength) {
+            const int sec_shift = damping - ulog2(sec_strength);
+            do {
+                for (int x = 0; x < w; x++) {
+                    const int px = dst[x];
+                    int sum = 0;
+                    int max = px, min = px;
+                    int pri_tap_k = pri_tap;
+                    for (int k = 0; k < 2; k++) {
+                        const int off1 = dav1d_cdef_directions[dir + 2][k]; // dir
+                        const int p0 = tmp[x + off1];
+                        const int p1 = tmp[x - off1];
+                        sum += pri_tap_k * constrain(p0 - px, pri_strength, pri_shift);
+                        sum += pri_tap_k * constrain(p1 - px, pri_strength, pri_shift);
+                        // if pri_tap_k == 4 then it becomes 2 else it remains 3
+                        pri_tap_k = (pri_tap_k & 3) | 2;
+                        min = umin(p0, min);
+                        max = imax(p0, max);
+                        min = umin(p1, min);
+                        max = imax(p1, max);
+                        const int off2 = dav1d_cdef_directions[dir + 4][k]; // dir + 2
+                        const int off3 = dav1d_cdef_directions[dir + 0][k]; // dir - 2
+                        const int s0 = tmp[x + off2];
+                        const int s1 = tmp[x - off2];
+                        const int s2 = tmp[x + off3];
+                        const int s3 = tmp[x - off3];
+                        // sec_tap starts at 2 and becomes 1
+                        const int sec_tap = 2 - k;
+                        sum += sec_tap * constrain(s0 - px, sec_strength, sec_shift);
+                        sum += sec_tap * constrain(s1 - px, sec_strength, sec_shift);
+                        sum += sec_tap * constrain(s2 - px, sec_strength, sec_shift);
+                        sum += sec_tap * constrain(s3 - px, sec_strength, sec_shift);
+                        min = umin(s0, min);
+                        max = imax(s0, max);
+                        min = umin(s1, min);
+                        max = imax(s1, max);
+                        min = umin(s2, min);
+                        max = imax(s2, max);
+                        min = umin(s3, min);
+                        max = imax(s3, max);
+                    }
+                    dst[x] = iclip(px + ((sum - (sum < 0) + 8) >> 4), min, max);
+                }
+                dst += PXSTRIDE(dst_stride);
+                tmp += tmp_stride;
+            } while (--h);
+        } else { // pri_strength only
+            do {
+                for (int x = 0; x < w; x++) {
+                    const int px = dst[x];
+                    int sum = 0;
+                    int pri_tap_k = pri_tap;
+                    for (int k = 0; k < 2; k++) {
+                        const int off = dav1d_cdef_directions[dir + 2][k]; // dir
+                        const int p0 = tmp[x + off];
+                        const int p1 = tmp[x - off];
+                        sum += pri_tap_k * constrain(p0 - px, pri_strength, pri_shift);
+                        sum += pri_tap_k * constrain(p1 - px, pri_strength, pri_shift);
+                        pri_tap_k = (pri_tap_k & 3) | 2;
+                    }
+                    dst[x] = px + ((sum - (sum < 0) + 8) >> 4);
+                }
+                dst += PXSTRIDE(dst_stride);
+                tmp += tmp_stride;
+            } while (--h);
         }
-        dst += PXSTRIDE(dst_stride);
-        tmp += tmp_stride;
+    } else { // sec_strength only
+        assert(sec_strength);
+        const int sec_shift = damping - ulog2(sec_strength);
+        do {
+            for (int x = 0; x < w; x++) {
+                const int px = dst[x];
+                int sum = 0;
+                for (int k = 0; k < 2; k++) {
+                    const int off1 = dav1d_cdef_directions[dir + 4][k]; // dir + 2
+                    const int off2 = dav1d_cdef_directions[dir + 0][k]; // dir - 2
+                    const int s0 = tmp[x + off1];
+                    const int s1 = tmp[x - off1];
+                    const int s2 = tmp[x + off2];
+                    const int s3 = tmp[x - off2];
+                    const int sec_tap = 2 - k;
+                    sum += sec_tap * constrain(s0 - px, sec_strength, sec_shift);
+                    sum += sec_tap * constrain(s1 - px, sec_strength, sec_shift);
+                    sum += sec_tap * constrain(s2 - px, sec_strength, sec_shift);
+                    sum += sec_tap * constrain(s3 - px, sec_strength, sec_shift);
+                }
+                dst[x] = px + ((sum - (sum < 0) + 8) >> 4);
+            }
+            dst += PXSTRIDE(dst_stride);
+            tmp += tmp_stride;
+        } while (--h);
     }
 }
 
@@ -170,7 +210,7 @@ cdef_filter_block_c(pixel *dst, const ptrdiff_t dst_stride,
 static void cdef_filter_block_##w##x##h##_c(pixel *const dst, \
                                             const ptrdiff_t stride, \
                                             const pixel (*left)[2], \
-                                            /*const*/ pixel *const top[2], \
+                                            const pixel *const top, \
                                             const int pri_strength, \
                                             const int sec_strength, \
                                             const int dir, \
@@ -178,8 +218,8 @@ static void cdef_filter_block_##w##x##h##_c(pixel *const dst, \
                                             const enum CdefEdgeFlags edges \
                                             HIGHBD_DECL_SUFFIX) \
 { \
-    cdef_filter_block_c(dst, stride, left, top, w, h, pri_strength, sec_strength, \
-                        dir, damping, edges HIGHBD_TAIL_SUFFIX); \
+    cdef_filter_block_c(dst, stride, left, top, pri_strength, sec_strength, \
+                        dir, damping, w, h, edges HIGHBD_TAIL_SUFFIX); \
 }
 
 cdef_fn(4, 4);
